@@ -24,10 +24,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 /* global prototype definitions */
-extern char *getlamer(struct channel *);
 extern char *safeban(struct channel *, char *, char *);
 extern char *getfirsthostless(struct channel *, char *);
-extern char *isnickbk(struct channel *, char *);
 extern char *getkickreason(char *);
 extern char *deltopic(struct channel *, int, char *);
 extern int partcount(struct channel *, char *, char *);
@@ -72,7 +70,6 @@ extern int isonchan(struct channel *, char *);
 extern int issameuser(struct channel *, char *, char *);
 extern int isopnow(struct channel *, char *);
 extern int nickchange(struct channel *, char *, char *);
-extern int iswordbk(struct channel *, char *);
 extern int isflood(struct channel *, char *, char *, char *);
 extern int isvoicenow(struct channel *, char *);
 extern int haschanflags(struct channel *, char*);
@@ -111,14 +108,14 @@ extern int getondiskjoincount(struct channel *, char *);
 extern int getrandommsg(struct channel *, char *, int, int, char *, int);
 extern int getjoincount(struct channel *, char *);
 extern int delserver(char *, int, int, int, int, int);
-extern int deladverts(struct channel *, char *);
+extern int deladverts(struct channel *, char *, int);
 extern int setpassword(struct channel *, char *, char *);
 extern int logoff(struct channel *, char *);
 extern int logon(struct channel *, struct botuser *, char *, char *, int);
 extern int timed_cmp(struct timed *, struct timed *);
 extern int clonechannel(char *, char *);
 extern int isfriend(struct channel *, char *, char *);
-extern int deltachanflags(struct channel *, char *, char);
+extern int deltaflags(struct channel *, char *, char);
 extern int isvalidchannel(char *);
 extern int hasuserflags(struct channel *, char *, char *, char *);
 extern unsigned int getrandom(unsigned int);
@@ -136,14 +133,16 @@ extern struct server *getserver(void);
 extern struct advert *addnewadvert(struct channel *, char *, char *);
 extern struct maskstruct *addnewmask(char *);
 extern struct maskstruct *addnewpermban(struct channel *, char *);
-extern struct maskstruct *addnewnickbk(struct channel *, char *, char *);
-extern struct maskstruct *addnewwordbk(struct channel *, char *);
+extern struct maskstruct *addnewnickbk(struct channel *, char *, char *, char *);
+extern struct maskstruct *addnewwordbk(struct channel *, char *, char *, char *);
 extern struct maskstruct *addnewtrusted(char *);
 extern struct maskstruct *addnewservice(char *);
 extern struct maskstruct *addnewkickreason(char *);
 extern struct maskstruct *addnewusermask(struct channel *, char *, char *);
 extern struct maskstruct *addnewadminmask(char *, char *);
 extern struct maskstruct *editmask(struct maskstruct *, char *, char *);
+extern struct maskstruct *isnickbk(struct channel *, char *);
+extern struct maskstruct *iswordbk(struct channel *, char *);
 extern struct pubmsg *addnewpubmsg(struct channel *, time_t, char *, char *, char *);
 extern struct part *addnewpart(struct channel *, time_t, char *, char *);
 extern struct join *addnewjoin(struct channel *, time_t, char *, char *);
@@ -194,6 +193,7 @@ extern void isjoined(struct channel *);
 extern void changetobek(struct channel *, char *, int);
 extern void setnick(void);
 extern void setumode(void);
+extern void sendqauth(void);
 extern void syncvoices(struct channel *);
 extern void syncops(struct channel *);
 extern void syncbans(struct channel *);
@@ -220,7 +220,7 @@ extern void quit(void);
 extern void parsectcp(char *, char *, int, char *);
 extern void nstats(struct channel *, char *, int, char *);
 extern void wall(struct channel *, char *, char *, char *);
-extern void cleanautheds(struct channel *);
+extern void cleanautheds(struct channel *, int);
 extern void listchanusers(void);
 extern void timed_del(struct timed *);
 extern void mmode_del(struct channel *, struct mmode *);
