@@ -369,9 +369,9 @@ int registerconnection()
 			strncpy(mystate->user, pwd->pw_name, USERLEN);
 		}
 		ptr = mystate->user;
-		while(*ptr == '~') {
-			/* server has prefixed our ident with ~
-			   strip ~ before saving to avoid double ~ */
+		while(*ptr == '~' || *ptr == '^') {
+			/* server has prefixed our ident with ~ or ^,
+			   strip ~ or ^ before saving */
 			ptr++;
 		}
 		strncpy(newuser, ptr, USERLEN);
