@@ -79,6 +79,8 @@ enum NTHMODE_RETURN_VALUES {
 	OER_NTHMODE_DVOICE,
 	OER_NTHMODE_BAN,
 	OER_NTHMODE_UNBAN,
+	OER_NTHMODE_CHANMODE_WITHOUT_PARAMS,
+	OER_NTHMODE_CHANMODE_WITH_PARAMS,
 	OER_NTHMODE_UNKNOWN
 };
 
@@ -256,6 +258,7 @@ enum OER_TIMED_PRIORITIES {
 #define OER_ACTIONS_PER_MAINLOOP 2
 #define OER_KICKS_PER_MAINLOOP 1
 #define OER_KICK_INTERVAL 2
+#define OER_GN_INTERVAL 10
 #define OER_NICKS_PER_USERHOST 5
 /* multiple kicks aren't supported for backwards compatibility */
 #define OER_NICKS_PER_KICK 1
@@ -330,10 +333,12 @@ enum OER_TIMED_PRIORITIES {
 #define USERHOSTLEN HOSTLEN + USERLEN + NICKLEN + 2
 #define TOPICLEN 160
 #define OER_MAX_LIST_LINES 5
+#define OER_UNKNOWN "unknown"
 #define OER_HELP "For a complete command reference go to http://oer.equnet.org/help.html"
 #define DEFAULT_NICKBK_MESSAGE "prohibited nick"
+#define DEFAULT_WORDBK_MESSAGE "word ban-kick triggered, have a nice life..."
 #define OER_REALNAME "http://oer.equnet.org/help.html"
-#define OER_COPYRIGHT1 "oer version 1.0-46, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
+#define OER_COPYRIGHT1 "oer version 1.0-47, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
 This is free software, and you are welcome to redistribute it under\n\
@@ -367,6 +372,14 @@ struct locked {
 	char host[USERHOSTLEN + 1];
 };
 
+struct qauth {
+	int hasauth;
+	int authed;
+	char q[USERHOSTLEN + 1];
+	char name[NICKLEN + 1];
+	char password[TINYSTRINGLEN + 1];
+};
+
 struct chanuser {
 	char *nick;
 	char *userhost;
@@ -461,7 +474,8 @@ struct part {
 
 struct maskstruct {
 	char *mask;
-	char *optstring;
+	char *optstring1;
+	char *optstring2;
 	struct maskstruct *prev;
 	struct maskstruct *next;
 };
@@ -511,6 +525,8 @@ struct channel {
 	int allhostsknown;
 	time_t rejoin_at;
 	time_t last_quote;
+	time_t join_ts;
+	time_t last_userhost;
 	FILE *seen;
 	FILE *last;
 	struct join *joins;
@@ -536,6 +552,7 @@ struct state {
 	char progname[STRINGLEN + 1];
 	char nick[NICKLEN + 1];
 	char altnick[NICKLEN + 1];
+	char getnick[NICKLEN + 1];
         char user[USERLEN + 1];
         char host[HOSTLEN + 1];
 	char mode[CHANLEN + 1];
@@ -562,8 +579,10 @@ struct state {
 	time_t postnj_checks_at;
 	time_t last_saved;
 	time_t last_linenoise;
+	time_t last_gn;
 	FILE *logfp;
 	FILE *logto;
+	struct qauth qauth;
 	struct authed *autheds;
 	struct botuser *admins;
 	struct channel *channels;
