@@ -189,6 +189,9 @@ int main(int argc, char *argv[])
 		oer_debug(OER_DEBUG_INFO, "main->loadall() failed\n");
 		exit(EXIT_FAILURE);
 	}
+	if(do_console) {
+                oer_debug(OER_DEBUG_INFO, "console mode is on (debug messages will be shown on screen)\n");
+        }
 	if(index(mystate->flags, (int)'l') != NULL) {
 		snprintf(stringbuffer, WRITE_BUFFER_LENGTH, "oer-%d.raw", getpid());
 		if((mystate->logfp = fopen(stringbuffer, "a+")) == NULL) {
