@@ -300,20 +300,30 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "433")) {
-		if(mystate->use_alt_nick) {
-#ifdef DEBUG
-                        oer_debug(OER_DEBUG_INFO, "parseirc(%s)->both our primary and secondary nicknames are taken, generating a random nick\n", command);
-#endif
-                        snprintf(mystate->nick, NICKLEN, "oer-%d", getrandom(9999));
-                        mystate->use_alt_nick = 0;
+		restpos = 0;
+		if((restpos = parse(restptr, restpos, " ", nick, NICKLEN, 0)) < 0) {
+			return;
+		}
+		/* check if we are still registering */
+                if(mystate->current_server->registered == OER_REGISTERCONNECTION_STATUS_NICK ||
+		   mystate->current_server->registered == OER_REGISTERCONNECTION_STATUS_USER) {
 			mystate->current_server->registered = OER_REGISTERCONNECTION_STATUS_BEGIN;
-                        return;
-                }
+			if(mystate->use_alt_nick) {
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, mystate->nick, mystate->altnick);
+				oer_debug(OER_DEBUG_INFO, "parseirc(%s)->both our primary and secondary nicknames are taken, generating a random nick\n", command);
 #endif
-		mystate->use_alt_nick = 1;
-		mystate->current_server->registered = OER_REGISTERCONNECTION_STATUS_BEGIN;
+				snprintf(mystate->nick, NICKLEN, "oer-%d", getrandom(9999));
+				mystate->use_alt_nick = 0;
+				return;
+			}
+#ifdef DEBUG
+			oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, nick, mystate->altnick);
+#endif
+			mystate->use_alt_nick = 1;
+			return;
+		}
+		/* no, must be a nick change that failed */
+		strncpy(mystate->nick, params, NICKLEN);
 		return;
 	}
 	if(!strcasecmp(command, "437")) {
@@ -752,23 +762,35 @@ void parseirc(char *serveroutput)
 			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on %s by %s!%s\n", mode, channel, temp_nick, userhost);
 #endif
 		}
-		/* lame way to keep chanmodes, i need to code a proper modeline parser */
-		if(!isme(temp_nick) && (index(mode, (int)'p') != NULL || index(mode, (int)'s') != NULL || index(mode, (int)'i') != NULL || index(mode, (int)'t') != NULL || index(mode, (int)'n') != NULL || index(mode, (int)'m') != NULL || index(mode, (int)'l') != NULL || index(mode, (int)'k') != NULL)) {
-			this->setchanmode = 1;			
-		}
 		i = 0;
 		bans = 0;
 		deops = 0;
 		while(1) {
-			nrestpos = parse(restptr, restpos, " ", target, STRINGLEN, 0);
-			if(nrestpos == restpos || nrestpos < 0) {
+			switch((modetype = nthmode(mode, i))) {
+			case OER_NTHMODE_OP:
+			case OER_NTHMODE_DOP:
+			case OER_NTHMODE_VOICE:
+			case OER_NTHMODE_DVOICE:
+			case OER_NTHMODE_BAN:
+			case OER_NTHMODE_UNBAN:
+			case OER_NTHMODE_CHANMODE_WITH_PARAMS:
+				nrestpos = parse(restptr, restpos, " ", target, STRINGLEN, 0);
+				if(nrestpos == restpos || nrestpos < 0) {
+					return;
+				}
+				restpos = nrestpos;
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->target is %s\n", target);
+#endif
 				break;
+			case OER_NTHMODE_UNKNOWN:
+				return;
 			}
-			restpos = nrestpos;
-			i++;
-			modetype = nthmode(mode, i);
 			switch(modetype) {
 			case OER_NTHMODE_OP:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_OP\n");
+#endif
 				changeuser(this, target, -1, 1, -1, -1);
 				if(isme(target)) {
 					this->i_am_op = 1;
@@ -784,12 +806,15 @@ void parseirc(char *serveroutput)
 						continue;
 					}
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, temp_nick, userhost);
+					oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, temp_nick, userhost);
 #endif
 					mmode_new(this, mystate->now, "-o", target);
 				}
 				break;
 			case OER_NTHMODE_DOP:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_DOP\n");
+#endif
 				changeuser(this, target, -1, 0, -1, -1);
 				if(isme(target)) {
 					this->i_am_op = 0;
@@ -805,12 +830,21 @@ void parseirc(char *serveroutput)
 				deops++;
 				break;
 			case OER_NTHMODE_VOICE:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_VOICE\n");
+#endif
 				changeuser(this, target, -1, -1, 1, -1);
 				break;
 			case OER_NTHMODE_DVOICE:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_DVOICE\n");
+#endif
 				changeuser(this, target, -1, -1, 0, -1);
 				break;
 			case OER_NTHMODE_BAN:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_BAN\n");
+#endif
 				snprintf(outstring, STRINGLEN, "%s!%s@%s", (mystate->use_alt_nick) ? mystate->altnick : mystate->nick, mystate->user, mystate->host);
 				if(wild_match(target, outstring)) {
 #ifdef DEBUG
@@ -832,6 +866,9 @@ void parseirc(char *serveroutput)
 				bans++;
 				break;
 			case OER_NTHMODE_UNBAN:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_UNBAN\n");
+#endif
 				if(this->i_am_op && haschanflags(this, "e") && ispermban(this, target)) {
 					if(isq(temp_nick) || isservice(temp_nick)) {
 						/* allow Q/service to unban */
@@ -842,17 +879,24 @@ void parseirc(char *serveroutput)
 					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on channel %s tried to remove the permban %s\n", temp_nick, channel, target);
 #endif
 					mmode_new(this, mystate->now, "+b", target);
-					if(!isserver(temp_nick)) {
-						snprintf(outstring, STRINGLEN, "do NOT remove %s ban from %s, it is in my permban list", target, channel);
-						sendreply(temp_nick, 0, 3, 0, outstring);
-
-					}
 				}
 				/* sync bans. if a user has set ban *@*.se and there is a
 				   permban *@*.swipnet.se, the latter has to be re-insert when
 				   the former is manually removed */
 				syncbans(this);
 				break;
+			case OER_NTHMODE_CHANMODE_WITHOUT_PARAMS:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_CHANMODE_WITHOUT_PARAMS\n");
+#endif
+				this->setchanmode = 1;
+				break;
+			case OER_NTHMODE_CHANMODE_WITH_PARAMS:
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_CHANMODE_WITH_PARAMS\n");
+#endif
+				this->setchanmode = 1;
+				break;
 			}
 			if((deops > OER_MASSMODE_LIMIT) && (index(userhost, '@') != NULL) && !isme(temp_nick)) {
 #ifdef DEBUG
@@ -869,10 +913,29 @@ void parseirc(char *serveroutput)
 					bans = 0;
 				}
 			}
+			i++;
 		}
 		return;
 	}
 	if(!strcasecmp(command, "TOPIC")) {
+		tpos = 0;
+		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+			return;
+		}
+		strncpy(channel, params, CHANLEN);
+		if((this = getchptr(channel)) == NULL) {
+			return;
+		}
+                if(haschanflags(this, "T")) {
+                        /* skip topic handling for this channel */
+                        return;
+                }
+                if(haschanflags(this, "t")) {
+			/* only allow topic changes through us */
+			if(!isme(nick)) {
+				this->topic_change = 1;
+			}
+		}
 		return;
 	}
 	if(!strcasecmp(command, "INVITE")) {
@@ -1077,21 +1140,21 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
         }
 	prevppos = ppos;
 	/* second is either the channel or first parameter to the command */
-        if((ppos = parse(commandline, ppos, " ", channel, CHANLEN, 0)) < 0) {
+        if((ppos = parse(commandline, ppos, " ", outstring, WRITE_BUFFER_LENGTH, 0)) < 0) {
                 return;
         }
-	if(isme(target)) {
-		tochan = 0;
-		if(!strlen(channel) || channel[0] != '#') {
+	tochan = (isme(target)) ? 0 : 1;
+	if(!tochan) {
+		if(!strlen(outstring) || outstring[0] != '#') {
 			/* no channel given, continue from here */
 			strncpy(channel, "<NONE>", CHANLEN);
 			restptr = commandline + prevppos;
 		} else {
 			/* advance to next parameter, skip channel */
+			strncpy(channel, outstring, CHANLEN);
 			restptr = commandline + ppos;
 		}
 	} else {
-		tochan = 1;
 		strncpy(channel, target, CHANLEN);
 		restptr = commandline + prevppos;
 	}
@@ -1398,7 +1461,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if((u = getrandomuser(this)) == NULL) {
 			return;
 		}
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "random kick by %s\n", nick);
+		snprintf(outstring, WRITE_BUFFER_LENGTH, "random kick by %s", nick);
 		kickuser(this, mystate->now, u->nick, outstring);
 		return;
 	case OER_WHICHCOMMAND_RANDOM_BANKICK:
@@ -1419,7 +1482,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if((u = getrandomuser(this)) == NULL) {
 			return;
 		}
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "random ban kick by %s\n", nick);
+		snprintf(outstring, WRITE_BUFFER_LENGTH, "random ban kick by %s", nick);
 		banuser(this, mystate->now - 1, u->nick);
 		kickuser(this, mystate->now, u->nick, outstring);
 		return;
@@ -1843,7 +1906,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->HELP\n");
 #endif
-		proceed = 0;
+		proceed = (isopa(userhost)) ? 1 : 0;
 		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
 			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
 				continue;
@@ -1864,7 +1927,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #endif
 		/* may be NULL */
 		this = getchptr(channel);
-		proceed = 0;
+                proceed = (isopa(userhost)) ? 1 : 0;
 		/* for some lists it is sufficient that the user
 		   is on one of our channels */
 		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
@@ -1955,7 +2018,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->SAVE\n");
 #endif
-		proceed = 0;
+                proceed = (isopa(userhost)) ? 1 : 0;
 		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
 			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
 				continue;
@@ -1974,7 +2037,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->FLUSH\n");
 #endif
-		proceed = 0;
+                proceed = (isopa(userhost)) ? 1 : 0;
 		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
 			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
 				continue;
@@ -1992,7 +2055,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->INFO\n");
 #endif
-		proceed = 0;
+                proceed = (isopa(userhost)) ? 1 : 0;
 		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
 			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
 				continue;
@@ -2011,7 +2074,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->UPTIME\n");
 #endif
-		proceed = 0;
+                proceed = (isopa(userhost)) ? 1 : 0;
 		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
 			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
 				continue;
@@ -2207,7 +2270,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
                 oer_debug(OER_DEBUG_INFO, "parsecommand->SSTATS\n");
 #endif
-                proceed = 0;
+                proceed = (isopa(userhost)) ? 1 : 0;
                 for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
                         if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
                                 continue;
